import NanoEvents from "nanoevents";
import { Connectivity, NetworkAdapter } from "../network/types";
export declare type CreateParams<Domain> = {
    network: NetworkAdapter<Domain>;
    onError?: (error: Error) => void;
    window?: Window;
};
export declare type FullDB<Domain> = {
    _internals: {
        canonData: Data<Domain>;
        debugConfig: DebugConfig;
        events: NanoEvents<Events<Domain>>;
        pendingTransactionCount: () => number;
        withPendingTransactions: (data: Data<Domain>) => Data<Domain>;
    };
    connectivity: () => Connectivity;
    create: <K extends keyof Domain>(kind: K, id: string, value: Domain[K]) => void;
    observe: (kind: keyof Domain, id: string) => () => void;
    update: <K extends keyof Domain>(kind: K, id: string, delta: Delta<Domain, K>) => void;
};
export declare type Data<Domain> = {
    [k in keyof Domain]: {
        [id: string]: {
            revision: string;
            value: Domain[k];
        } | undefined;
    };
};
export declare type Delta<Domain, K extends keyof Domain> = (prev: Domain[K]) => Domain[K];
export declare type DebugConfig = {
    failingWrites: boolean;
    pretendOffline: boolean;
    optimisticUIEnabled: boolean;
};
export declare type Events<Domain> = {
    change: [keyof Domain, string];
    "debug-config-changed": undefined;
    "pending-transaction-count-changed": undefined;
    "connectivity-changed": undefined;
};
export declare type Transaction<Domain, K extends keyof Domain = keyof Domain> = {
    kind: K;
    id: string;
} & TransactionAction<Domain, K>;
export declare type TransactionAction<Domain, K extends keyof Domain = keyof Domain> = {
    delta: Delta<Domain, K>;
} | {
    creation: Domain[K];
};
